/*!
   \file scalar_algorithm_redistribute.cpp
   \brief Algorithm for redistribute a scalar.
 */
#include "scalar_algorithm_redistribute.hpp"
#include "redistribute_scalar.hpp"
#include "scalar.hpp"
#include "grid.hpp"
#include "grid_transformation_factory_impl.hpp"

namespace xios
{

  shared_ptr<CGenericAlgorithmTransformation> CScalarAlgorithmRedistribute::create(bool isSource, CGrid* gridDst, CGrid* gridSrc,
                                                               CTransformation<CScalar>* transformation,
                                                               int elementPositionInGrid,
                                                               std::map<int, int>& elementPositionInGridSrc2ScalarPosition,
                                                               std::map<int, int>& elementPositionInGridSrc2AxisPosition,
                                                               std::map<int, int>& elementPositionInGridSrc2DomainPosition, 
                                                               std::map<int, int>& elementPositionInGridDst2ScalarPosition,
                                                               std::map<int, int>& elementPositionInGridDst2AxisPosition,
                                                               std::map<int, int>& elementPositionInGridDst2DomainPosition)
  TRY
  {
    std::vector<CScalar*> scalarListDestP = gridDst->getScalars();
    std::vector<CScalar*> scalarListSrcP  = gridSrc->getScalars();

    CRedistributeScalar* redistributeScalar = dynamic_cast<CRedistributeScalar*> (transformation);
    int scalarDstIndex = elementPositionInGridDst2ScalarPosition[elementPositionInGrid];
    int scalarSrcIndex = elementPositionInGridSrc2ScalarPosition[elementPositionInGrid];

    return make_shared<CScalarAlgorithmRedistribute>(isSource, scalarListDestP[scalarDstIndex], scalarListSrcP[scalarSrcIndex], redistributeScalar);
  }
  CATCH

  bool CScalarAlgorithmRedistribute::dummyRegistered_ = CScalarAlgorithmRedistribute::registerTrans();
  bool CScalarAlgorithmRedistribute::registerTrans()
  TRY
  {
    return CGridTransformationFactory<CScalar>::registerTransformation(TRANS_REDISTRIBUTE_SCALAR, create);
  }
  CATCH

  CScalarAlgorithmRedistribute::CScalarAlgorithmRedistribute(bool isSource, CScalar* scalarDestination, CScalar* scalarSource, CRedistributeScalar* redistributeScalar)
  : CAlgorithmTransformationTransfer(isSource)
  TRY
  {


    scalarDestination->checkAttributes() ;
  }
  CATCH


}
